@@ -16,7 +16,8 @@ from openstack import resource
 
 
 class FloatingIP(resource.Resource):
-    name_attribute = "floating_ip_address"
+    id_attribute = "floating_ip_address"
+    name_attribute = None
     resource_name = "floating ip"
     resource_key = 'floatingip'
     resources_key = 'floatingips'
@@ -29,7 +30,6 @@ class FloatingIP(resource.Resource):
     allow_update = True
     allow_delete = True
     allow_list = True
-    patch_update = False
 
     # Properties
     fixed_ip_address = resource.prop('fixed_ip_address')
@@ -46,6 +46,8 @@ class FloatingIP(resource.Resource):
             'fields': cls.id_attribute,
         }
         info = cls.list(session, **args)
-        if len(info) > 0:
-            return info[0]
-        raise exceptions.ResourceNotFound("No available floating ips exist.")
+        try:
+            return next(info)
+        except StopIteration:
+            msg = "No available floating ips exist."
+            raise exceptions.ResourceNotFound(msg)
