@@ -90,19 +90,20 @@ class prop(object):
         Far Away
     """
 
-    def __init__(self, name, alias=None, type=None):
+    def __init__(self, name, alias=None, type=None, default=None):
         self.name = name
         self.type = type
         self.alias = alias
+        self.default = default
 
     def __get__(self, instance, owner):
         try:
             value = instance._attrs[self.name]
         except KeyError:
             try:
-                return instance._attrs[self.alias]
+                value = instance._attrs[self.alias]
             except KeyError:
-                raise AttributeError('Unset property: %s' % self.name)
+                return self.default
 
         if self.type and not isinstance(value, self.type):
             value = self.type(value)
@@ -128,7 +129,7 @@ class prop(object):
             try:
                 del instance._attrs[self.alias]
             except KeyError:
-                raise AttributeError('Unset property: %s' % self.name)
+                pass
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -165,8 +166,6 @@ class Resource(collections.MutableMapping):
     #: Allow head operation for this resource.
     allow_head = False
 
-    patch_update = True
-
     def __init__(self, attrs=None, loaded=False):
         if attrs is None:
             attrs = {}
@@ -364,6 +363,7 @@ class Resource(collections.MutableMapping):
         data = self.head_data_by_id(session, self.id, path_args=self)
         self._attrs.update(data)
         self._loaded = True
+        return self
 
     @classmethod
     def update_by_id(cls, session, r_id, attrs, path_args=None):
@@ -381,10 +381,7 @@ class Resource(collections.MutableMapping):
         else:
             url = cls.base_path
         url = utils.urljoin(url, r_id)
-        if cls.patch_update:
-            resp = session.patch(url, service=cls.service, json=body).body
-        else:
-            resp = session.put(url, service=cls.service, json=body).body
+        resp = session.patch(url, service=cls.service, json=body).body
 
         if cls.resource_key:
             resp = resp[cls.resource_key]
@@ -428,57 +425,90 @@ class Resource(collections.MutableMapping):
 
     @classmethod
     def list(cls, session, limit=None, marker=None, path_args=None, **params):
-        """Get a list of resources as an array of objects."""
+        """Return a generator that will page through results of GET requests.
 
-        # NOTE(jamielennox): Is it possible we can return a generator from here
-        # and allow us to keep paging rather than limit and marker?
+        This method starts at `limit` and `marker` (both defaulting to None),
+        advances the marker to the last item received, and continues paging
+        until no results are returned.
+        """
         if not cls.allow_list:
             raise exceptions.MethodNotSupported('list')
 
-        filters = {}
+        more_data = True
 
-        if limit:
-            filters['limit'] = limit
-        if marker:
-            filters['marker'] = marker
+        while more_data:
+            filters = {}
 
-        if path_args:
-            url = cls.base_path % path_args
-        else:
-            url = cls.base_path
-        if filters:
-            url = '%s?%s' % (url, url_parse.urlencode(filters))
+            if limit:
+                filters['limit'] = limit
+            if marker:
+                filters['marker'] = marker
 
-        resp = session.get(url, service=cls.service, params=params).body
+            if path_args:
+                url = cls.base_path % path_args
+            else:
+                url = cls.base_path
+            if filters:
+                url = '%s?%s' % (url, url_parse.urlencode(filters))
 
-        if cls.resources_key:
-            resp = resp[cls.resources_key]
+            resp = session.get(url, service=cls.service, params=params).body
 
-        return [cls.existing(**data) for data in resp]
+            if cls.resources_key:
+                resp = resp[cls.resources_key]
+
+            # TODO(briancurtin): Although there are a few different ways
+            # across services, we can know from a response if it's the end
+            # without doing an extra request to get an empty response.
+            # Resources should probably carry something like a `_should_page`
+            # method to handle their service's pagination style.
+            if not resp:
+                more_data = False
+
+            for data in resp:
+                value = cls.existing(**data)
+                marker = value.id
+                yield value
 
     @classmethod
-    def find(cls, session, name_or_id, path_args=None, id_only=True):
+    def find(cls, session, name_or_id, path_args=None):
         """Find a resource by name or id as an instance."""
         try:
-            params = {cls.id_attribute: name_or_id}
-            if id_only:
-                params['fields'] = cls.id_attribute
-            info = cls.list(session, path_args=path_args, **params)
-            if len(info) == 1:
-                return info[0]
+            args = {
+                cls.id_attribute: name_or_id,
+                'fields': cls.id_attribute,
+                'path_args': path_args,
+            }
+            info = cls.list(session, **args)
+            # If there is exactly one result available, return it.
+            result = None
+            try:
+                result = next(info)
+                next(info)
+            except StopIteration:
+                if result is not None:
+                    return result
         except exceptions.HttpException:
             pass
+
         if cls.name_attribute:
-            params = {cls.name_attribute: name_or_id}
-            if id_only:
-                params['fields'] = cls.id_attribute
+            params = {cls.name_attribute: name_or_id,
+                      'fields': cls.id_attribute}
             info = cls.list(session, path_args=path_args, **params)
-            if len(info) == 1:
-                return info[0]
-            if len(info) > 1:
+            result = None
+            # Take the first value as our result. If another value is,
+            # available then raise DuplicateResource.
+            try:
+                result = next(info)
+                next(info)
                 msg = "More than one %s exists with the name '%s'."
                 msg = (msg % (cls.get_resource_name(), name_or_id))
                 raise exceptions.DuplicateResource(msg)
+            except StopIteration:
+                # We got here because `info` either gave us the result
+                # or it was empty.
+                if result is not None:
+                    return result
+
         msg = ("No %s with a name or ID of '%s' exists." %
                (cls.get_resource_name(), name_or_id))
         raise exceptions.ResourceNotFound(msg)
